

# This example is to apply the high-dimensional knockoff filter based on 
# two steps  proposed in Candes's paper 2016 to compute the FDR and Power.

# see the paper titled by "A knockoff filter for high-dimensional selective inference"
# by R. F. Barber and E. J. Candes in 2016


#  control the FDR for Ultra-high dimensional regression using Knockfilter
#  In the first step: screening step using the Lasso: biglasso
#  In the scend step: variable selection based on the spliting data and recycling data


# List of R functions for HDKnockoff:

##########################################################
## According to high-Knockoff paper, we first splite the sample into two disjoint group
sample.Splite = function(X, y){
  n   = nrow(X)
  p   = ncol(X)
  n0  = n/2
  n1  = n-n0
  full.sample   = c(1:n)
  scr.sample    = sample(full.sample,n0)
  sel.sample    = setdiff(full.sample,scr.sample)
  X_0           = X[scr.sample,]
  y_0           = y[scr.sample]
  X_1           = X[sel.sample,]
  y_1           = y[sel.sample]
  obj  = list(X0 = X_0, y0 = y_0, X1 = X_1, y1 = y_1)
  return(obj)
}

####################################################################

## In the screening step, we use the lasso to select a set
sel.set = function(X0, y0, n){
  n1 = n-nrow(X0)
  ## fit.lasso1 = scalreg(X0, y0, lam0 = NULL)     ## using the scalreg package
  ## set0  = order(abs(fit.lasso1$coefficients),decreasing=T)
  
  fit.lasso1 = cv.glmnet(X0, y0, dfmax = n/4)     ## using the glmnet package
  set0  = order(abs(coef(fit.lasso1)[-1,]),decreasing=T)
  scr.set      = set0[1:(n1/2)]
  scr.set.recy = set0[1:n1]
  obj = list(scr.set=scr.set, scr.set.recy=scr.set.recy)
  return(obj)
}

######################################################################
## After screening, we can get the selected variable or a reduced model
## y1 = X1_{\hat{S}_0}\beta^{p}+e1
sel.variable = function(scr.set, X0, X1, beta.true){
  X0.sel = X0[,scr.set]
  X1.sel = X1[,scr.set]
  beta.sel = beta.true[scr.set]
  obj  = list(X0.sel = X0.sel, X1.sel = X1.sel, beta.sel = beta.sel)
  return(obj)
}

######################################################################

###########################################################################
# Part II: additional functions
# Part II 1) fdr: calculating fdr

Hfdr <- function(S, beta.S) {
  Hfdp = sum(beta.S[S] == 0)/max(1, length(S))
  return(Hfdp)
}
###########################################################################

###########################################################################
# Part II: additional functions
# Part II 2) pow: calculating power

Hpow <- function(S, beta.S) {
  Htdp = sum(beta.S[S] != 0)/sum(beta.S != 0)
  return(Htdp)
}
# The end of the master R file
###########################################################################

  



